-- Sistemas Integrados TEKNISA
-- Apurao de ponto
--
-- Nome: 0000_STPTEKO_CDA_TOTALIZA.sql
--
-- Descrio:
-- Realiza a totalizao de faltas dos operadores sem controle de jornada, atravs da verificao das CDAs lanadas.
--
-- Parmetros:
-- P_NRORG - Nmero da organizao
-- P_CDEMPRESA - Cdigo da empresa
-- P_CDFUNC - Cdigo do funcionrio
-- P_DTPONTO - Data do ponto
-- 
-- Criao:
-- 08/02/2013 Newton Neves dos Santos

CREATE OR REPLACE 
PROCEDURE FPA_CDA_TOTALIZA
  (P_NRORG IN FPA_PONTO.NRORG%TYPE,
   P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
   P_CDFUNC IN FUNCIONARIO.CDFUNC%TYPE,
   P_DTPONTO IN BATIDAPONTO.DTBATIDAPTO%TYPE) 

IS
   V_CDEMPRESAFOLHA FUNCIONARIO.CDEMPRESAFOLHA%TYPE;
   V_CDTPADMISSAO FUNCIONARIO.CDTPADMISSAO%TYPE;
   V_NRPONTO NUMBER;
   V_DTPONTO SERVPREST.DTSERVPREST%TYPE;
   V_DIASEMPTO VARCHAR2(01);
   V_ST_NRPONTO NOVOCODIGO.NRSEQUENCIAL%TYPE;
   V_FALTA_NA_SEMANA BOOLEAN;
   V_DTINICIOAUX FPA_PERIODOAPURAPONTO.DTINICIALAPURAPONTO%TYPE;
   V_DTFIMAUX FPA_PERIODOAPURAPONTO.DTFINALAPURAPONTO%TYPE;
   
   V_DTINICIALAPURAPONTO FPA_PERIODOAPURAPONTO.DTINICIALAPURAPONTO%TYPE;
   V_DTFINALAPURAPONTO FPA_PERIODOAPURAPONTO.DTFINALAPURAPONTO%TYPE;
   V_NRPERIODOAPURAPONTO FPA_PERIODOAPURAPONTO.NRPERIODOAPURAPONTO%TYPE;
   
   V_CDCLIENTE SERVPREST.CDCLIENTE%TYPE;
   V_IDSERVPREST SERVPREST.IDSERVPREST%TYPE;
   V_DTSERVPREST SERVPREST.DTSERVPREST%TYPE;
   V_NRSERVPREST SERVPREST.NRSERVPREST%TYPE;
   
   V_QTDIASDESCONTOFALTA FPA_TOTALIZAPONTO.QTDIASDESCONTOFALTA%TYPE;
   V_QTDIASDESCONTODSR FPA_TOTALIZAPONTO.QTDIASDESCONTODSR%TYPE;

   CURSOR C_FPA_CDA_TOTAL IS 
       SELECT D.DTDATA, TO_CHAR(D.DTDATA, 'D'), NVL(S.IDSERVPREST, 'ATIVIDADE_CLIENTE'), S.DTSERVPREST, S.NRSERVPREST, S.CDCLIENTE
         FROM SERVPREST S, (SELECT P_NRORG AS NRORG, P_CDEMPRESA AS CDEMPRESA, P_CDFUNC AS CDFUNC, DTDATA 
                              FROM DATA
                             WHERE DTDATA BETWEEN V_DTINICIOAUX AND V_DTFIMAUX
                               AND TO_CHAR(DTDATA, 'D') <> '1') D
        WHERE D.NRORG = S.NRORG(+)
          AND D.CDEMPRESA = S.CDEMPRESA(+)
          AND D.CDFUNC = S.CDFUNC(+)
          AND D.DTDATA = S.DTSERVPREST(+)
        UNION ALL 
       SELECT D.DTDATA, TO_CHAR(D.DTDATA, 'D'), NVL(S.IDSERVPREST, 'ATIVIDADE_CLIENTE'), S.DTSERVPREST, S.NRSERVPREST, S.CDCLIENTE
         FROM SERVPREST S, (SELECT P_NRORG AS NRORG, P_CDEMPRESA AS CDEMPRESA, P_CDFUNC AS CDFUNC, DTDATA 
                              FROM DATA
                             WHERE DTDATA BETWEEN V_DTINICIOAUX AND V_DTFIMAUX
                               AND TO_CHAR(DTDATA, 'D') = '1') D
        WHERE D.NRORG = S.NRORG(+)
          AND D.CDEMPRESA = S.CDEMPRESA(+)
          AND D.CDFUNC = S.CDFUNC(+)
          AND D.DTDATA = S.DTSERVPREST(+)
          AND S.IDSERVPREST(+) = 'FALTA_NAO_JUSTIFICADA'
          AND S.NRSERVPREST(+) = 'FALTA'
        ORDER BY DTDATA;

BEGIN
/* ##################################################################################################################### */
/* ################################## TOTALIZAO DO PONTO NO PERODO (INCIO) ######################################### */
/* ##################################################################################################################### */

   SELECT FUN.CDEMPRESAFOLHA, FUN.CDTPADMISSAO 
     INTO V_CDEMPRESAFOLHA, V_CDTPADMISSAO
     FROM FUNCIONARIO FUN
    WHERE FUN.CDFUNC = P_CDFUNC
      AND FUN.CDEMPRESA = P_CDEMPRESA;
      
   IF V_CDTPADMISSAO <> 'J' THEN
      RETURN;
   END IF;

    /* Verificao do perodo de apurao de acordo com a tabela FPA_PERIODOAPURAPONTO */
   SELECT P.NRPERIODOAPURAPONTO, P.DTINICIALAPURAPONTO, P.DTFINALAPURAPONTO
     INTO V_NRPERIODOAPURAPONTO, V_DTINICIALAPURAPONTO, V_DTFINALAPURAPONTO
     FROM FPA_PERIODOAPURAPONTO P, FUNCIONARIO F
    WHERE F.CDEMPRESA = P_CDEMPRESA
      AND F.CDFUNC = P_CDFUNC
      AND F.NRORG = P_NRORG
      AND P.CDTPADMISSAO = V_CDTPADMISSAO
      AND P.DTINICIALAPURAPONTO <= P_DTPONTO
      AND P.DTFINALAPURAPONTO >= P_DTPONTO;

   DELETE 
     FROM FPA_TOTALIZAPONTO
    WHERE NRORG = P_NRORG
      AND CDEMPRESA = P_CDEMPRESA
      AND CDFUNC = P_CDFUNC
      AND NRPERIODOAPURAPONTO = V_NRPERIODOAPURAPONTO;

   /* Para tratamento do desconto do DSR em caso de falta no abonada, o perodo considerado 
      sempre iniciar numa segunda-feira e terminar num domingo */ 
   IF TO_CHAR(V_DTINICIALAPURAPONTO, 'D') = '1'  THEN /* 1 -> Domingo */ 
      V_DTINICIOAUX := TRUNC(V_DTINICIALAPURAPONTO-6);
   ELSE
      V_DTINICIOAUX := TRUNC(V_DTINICIALAPURAPONTO+(2-TO_NUMBER(TO_CHAR(V_DTINICIALAPURAPONTO, 'D'))));
   END IF;   
   V_DTFIMAUX := TRUNC(V_DTFINALAPURAPONTO+(8-TO_NUMBER(TO_CHAR(V_DTFINALAPURAPONTO, 'D'))));
   
   V_QTDIASDESCONTOFALTA := 0;
   V_QTDIASDESCONTODSR := 0;
   V_FALTA_NA_SEMANA := FALSE;

   OPEN C_FPA_CDA_TOTAL;
   LOOP
      FETCH C_FPA_CDA_TOTAL INTO V_DTPONTO, V_DIASEMPTO, V_IDSERVPREST, V_DTSERVPREST, V_NRSERVPREST, V_CDCLIENTE;
      EXIT WHEN C_FPA_CDA_TOTAL%NOTFOUND;
      
      IF (V_DTPONTO >= V_DTINICIALAPURAPONTO) AND (V_DTPONTO <= V_DTFINALAPURAPONTO) THEN 
         IF (V_IDSERVPREST = 'FALTA_NAO_JUSTIFICADA') AND (V_DIASEMPTO <> '1') THEN /* 1 -> Domingo */
            V_QTDIASDESCONTOFALTA := V_QTDIASDESCONTOFALTA + 1;
         END IF;   
         
         IF (V_DIASEMPTO = '1') AND V_FALTA_NA_SEMANA THEN 
            V_QTDIASDESCONTODSR := V_QTDIASDESCONTODSR + 1;
         END IF;
      END IF;
      
      IF (V_IDSERVPREST = 'FALTA_NAO_JUSTIFICADA') AND (V_DIASEMPTO <> '1') THEN /* 1 -> Domingo */
         V_FALTA_NA_SEMANA := TRUE;
      END IF;   
      
      IF (V_DIASEMPTO = '1') THEN 
         IF V_FALTA_NA_SEMANA THEN         
            V_FALTA_NA_SEMANA := FALSE;

            IF V_NRSERVPREST IS NULL THEN
               INSERT 
                 INTO SERVPREST (NRORG, CDEMPRESA, CDFUNC, CDCLIENTE, DTSERVPREST, NRSERVPREST, IDSERVPREST, DSREFERENCIA, CDTIPOSERV)
               VALUES (P_NRORG, P_CDEMPRESA, P_CDFUNC, 'J26269316000177', V_DTPONTO, 'FALTA', 'FALTA_NAO_JUSTIFICADA', 'FALTA_NAO_JUSTIFICADA', '030');
            ELSE
               UPDATE SERVPREST
                  SET IDSERVPREST = 'FALTA_NAO_JUSTIFICADA'
                WHERE CDEMPRESA = P_CDEMPRESA
                  AND CDFUNC = P_CDFUNC
                  AND CDCLIENTE = V_CDCLIENTE
                  AND DTSERVPREST = V_DTSERVPREST
                  AND NRSERVPREST = V_NRSERVPREST
                  AND NRORG = P_NRORG;
            END IF;
         ELSIF (V_NRSERVPREST IS NOT NULL) AND (V_IDSERVPREST = 'FALTA_NAO_JUSTIFICADA') THEN
            DELETE FROM SERVPREST
             WHERE CDEMPRESA = P_CDEMPRESA
               AND CDFUNC = P_CDFUNC
               AND CDCLIENTE = V_CDCLIENTE
               AND DTSERVPREST = V_DTSERVPREST
               AND NRSERVPREST = V_NRSERVPREST
               AND NRORG = P_NRORG;
         END IF;
      END IF;

   END LOOP;
   CLOSE C_FPA_CDA_TOTAL;

   NOVO_CODIGO('FPA_TOTALIZAPONTO', V_ST_NRPONTO, 1, P_NRORG);
   V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
   
   INSERT INTO FPA_TOTALIZAPONTO (NRTOTALIZAPONTO, NRORG, CDEMPRESA, CDFUNC, CDEMPRESAFOLHA, NRPERIODOAPURAPONTO, QTDIASDESCONTOFALTA, QTDIASDESCONTODSR)
        VALUES (V_NRPONTO, P_NRORG, P_CDEMPRESA, P_CDFUNC, V_CDEMPRESAFOLHA, V_NRPERIODOAPURAPONTO, V_QTDIASDESCONTOFALTA, V_QTDIASDESCONTODSR);
   
/* ##################################################################################################################### */
/* ################################## TOTALIZAO DO PONTO NO PERODO (TRMINO) ######################################## */
/* ##################################################################################################################### */

   COMMIT;
END;
/